@@ -1,19 +1,19 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, codecs
+import codecs
+import re
 
+from calibre import force_unicode
 from calibre.ebooks.BeautifulSoup import BeautifulSoup
 from calibre.ebooks.chardet import xml_to_unicode
-from calibre.ebooks.metadata import string_to_authors, MetaInformation
-from calibre.utils.logging import default_log
+from calibre.ebooks.metadata import MetaInformation, string_to_authors
 from calibre.ptempfile import TemporaryFile
-from calibre import force_unicode
+from calibre.utils.logging import default_log
 from polyglot.builtins import iterkeys
 
 
@@ -80,7 +80,7 @@ def _get_comments(soup):
         date = date.replace('\u00a9', '').strip()
         # and pages often comes as '(\d+ pages)'
         pages = re.search(r'\d+', pages).group(0)
-        return 'Published %s, %s pages.' % (date, pages)
+        return f'Published {date}, {pages} pages.'
     except:
         pass
     return None
@@ -125,8 +125,9 @@ def _get_cover(soup, rdr):
             except:
                 ans = None
         if ans is not None:
-            from PIL import Image
             import io
+
+            from PIL import Image
             buf = io.BytesIO()
             try:
                 Image.open(io.BytesIO(ans)).convert('RGB').save(buf, 'JPEG')
