@@ -7,15 +7,14 @@ __docformat__ = 'restructuredtext en'
 
 import copy
 import traceback
+from contextlib import suppress
 
 from calibre import prints
 from calibre.constants import DEBUG
-from calibre.ebooks.metadata.book import (
-    ALL_METADATA_FIELDS, SC_COPYABLE_FIELDS, SC_FIELDS_COPY_NOT_NULL,
-    STANDARD_METADATA_FIELDS, TOP_LEVEL_IDENTIFIERS,
-)
+from calibre.ebooks.metadata.book import ALL_METADATA_FIELDS, SC_COPYABLE_FIELDS, SC_FIELDS_COPY_NOT_NULL, STANDARD_METADATA_FIELDS, TOP_LEVEL_IDENTIFIERS
 from calibre.library.field_metadata import FieldMetadata
-from calibre.utils.icu import lower as icu_lower, sort_key
+from calibre.utils.icu import lower as icu_lower
+from calibre.utils.icu import sort_key
 from calibre.utils.localization import ngettext
 from polyglot.builtins import iteritems, string_or_bytes
 
@@ -202,7 +201,7 @@ class Metadata:
         return iter(object.__getattribute__(self, '_data'))
 
     def has_key(self, key):
-        return key in object.__getattribute__(self, '_data')
+        return key in STANDARD_METADATA_FIELDS or key in object.__getattribute__(self, '_data')['user_metadata']
 
     def _evaluate_all_composites(self):
         custom_fields = object.__getattribute__(self, '_data')['user_metadata']
@@ -233,6 +232,11 @@ class Metadata:
         # are returning a "real" Metadata instance that has __get_attribute__.
         m = Metadata(None)
         object.__setattr__(m, '_data', copy.deepcopy(object.__getattribute__(self, '_data')))
+        # Also copy these two top-level attributes as they can appear in templates.
+        with suppress(AttributeError):
+            object.__setattr__(m, 'id', copy.copy(self.__getattribute__('id')))
+        with suppress(AttributeError):
+            object.__setattr__(m, 'has_cover', copy.copy(self.__getattribute__('has_cover')))
         return m
 
     def get(self, field, default=None):
@@ -516,7 +520,7 @@ class Metadata:
         '''
         def copy_not_none(dest, src, attr):
             v = getattr(src, attr, None)
-            if v not in (None, NULL_VALUES.get(attr, None)):
+            if v is not None and v != NULL_VALUES.get(attr, None):
                 setattr(dest, attr, copy.deepcopy(v))
 
         unknown = _('Unknown')
