@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 '''
 Read content from ereader pdb file with a 132 byte header created by Dropbook.
 '''
@@ -18,10 +16,9 @@ from calibre.ebooks import DRMError
 from calibre.ebooks.metadata.opf2 import OPFCreator
 from calibre.ebooks.pdb.ereader import EreaderError
 from calibre.ebooks.pdb.formatreader import FormatReader
-from polyglot.builtins import unicode_type, range
 
 
-class HeaderRecord(object):
+class HeaderRecord:
     '''
     The first record in the file is always the header record. It holds
     information related to the location of text, images, and so on
@@ -87,9 +84,9 @@ class Reader132(FormatReader):
 
     def get_image(self, number):
         if number < self.header_record.image_data_offset or number > self.header_record.image_data_offset + self.header_record.num_image_pages - 1:
-            return 'empty', ''
+            return 'empty', b''
         data = self.section_data(number)
-        name = data[4:4 + 32].strip('\x00')
+        name = data[4:4 + 32].strip(b'\x00').decode(self.encoding or 'cp1252')
         img = data[62:]
         return name, img
 
@@ -105,8 +102,7 @@ class Reader132(FormatReader):
         return self.decompress_text(number)
 
     def extract_content(self, output_dir):
-        from calibre.ebooks.pml.pmlconverter import footnote_to_html, sidebar_to_html
-        from calibre.ebooks.pml.pmlconverter import PML_HTMLizer
+        from calibre.ebooks.pml.pmlconverter import PML_HTMLizer, footnote_to_html, sidebar_to_html
 
         output_dir = os.path.abspath(output_dir)
 
@@ -114,11 +110,11 @@ class Reader132(FormatReader):
             os.makedirs(output_dir)
 
         title = self.mi.title
-        if not isinstance(title, unicode_type):
+        if not isinstance(title, str):
             title = title.decode('utf-8', 'replace')
-        html = u'<html><head><title>%s</title></head><body>' % title
+        html = '<html><head><title>%s</title></head><body>' % title
 
-        pml = u''
+        pml = ''
         for i in range(1, self.header_record.num_text_pages + 1):
             self.log.debug('Extracting text page %i' % i)
             pml += self.get_text_page(i)
