@@ -1,20 +1,20 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+#!/usr/bin/env python
 
-import re, sys
+
+import re
+import sys
 from collections import defaultdict
 
-from lxml.etree import tostring
-from lxml.html import (fragment_fromstring, document_fromstring,
-        tostring as htostring)
+from lxml.html import document_fromstring, fragment_fromstring
+from lxml.html import tostring as htostring
 
+from calibre.ebooks.readability.cleaners import clean_attributes, html_cleaner
 from calibre.ebooks.readability.htmls import build_doc, get_body, get_title, shorten_title
-from calibre.ebooks.readability.cleaners import html_cleaner, clean_attributes
+from polyglot.builtins import reraise
+
 
 def tounicode(tree_or_node, **kwargs):
-    kwargs['encoding'] = unicode
+    kwargs['encoding'] = str
     return htostring(tree_or_node, **kwargs)
 
 
@@ -33,6 +33,7 @@ REGEXES = {
     # skipFootnoteLink:    /^\s*(\[?[a-z0-9]{1,2}\]?|^|edit|citation needed)\s*$/i,
 }
 
+
 def describe(node, depth=1):
     if not hasattr(node, 'tag'):
         return "[%s]" % type(node)
@@ -47,6 +48,7 @@ def describe(node, depth=1):
         return name+' - '+describe(node.getparent(), depth-1)
     return name
 
+
 def to_int(x):
     if not x:
         return None
@@ -57,17 +59,21 @@ def to_int(x):
         return int(x[:-2]) * 12
     return int(x)
 
+
 def clean(text):
-    text = re.sub('\s*\n\s*', '\n', text)
+    text = re.sub('\\s*\n\\s*', '\n', text)
     text = re.sub('[ \t]{2,}', ' ', text)
     return text.strip()
 
+
 def text_length(i):
     return len(clean(i.text_content() or ""))
 
-class Unparseable(ValueError):
+
+class Unparsable(ValueError):
     pass
 
+
 class Document:
     TEXT_LENGTH_THRESHOLD = 25
     RETRY_LENGTH = 250
@@ -147,9 +153,9 @@ class Document:
                     continue  # try again
                 else:
                     return cleaned_article
-        except StandardError, e:
+        except Exception as e:
             self.log.exception('error getting summary: ')
-            raise Unparseable(str(e)), None, sys.exc_info()[2]
+            reraise(Unparsable, Unparsable(str(e)), sys.exc_info()[2])
 
     def get_article(self, candidates, best_candidate):
         # Now that we have the top candidate, look through its siblings for content that might also be related.
@@ -176,7 +182,7 @@ class Document:
 
                 if node_length > 80 and link_density < 0.25:
                     append = True
-                elif node_length < 80 and link_density == 0 and re.search('\.( |$)', node_content):
+                elif node_length < 80 and link_density == 0 and re.search(r'\.( |$)', node_content):
                     append = True
 
             if append:
@@ -189,7 +195,7 @@ class Document:
         sorted_candidates = sorted(candidates.values(), key=lambda x: x['content_score'], reverse=True)
         for candidate in sorted_candidates[:5]:
             elem = candidate['elem']
-            self.debug("Top 5 : %6.3f %s" % (candidate['content_score'], describe(elem)))
+            self.debug("Top 5 : {:6.3f} {}".format(candidate['content_score'], describe(elem)))
 
         if len(sorted_candidates) == 0:
             return None
@@ -250,7 +256,7 @@ class Document:
             candidate = candidates[elem]
             ld = self.get_link_density(elem)
             score = candidate['content_score']
-            self.debug("Candid: %6.3f %s link density %.3f -> %6.3f" % (score, describe(elem), ld, score*(1-ld)))
+            self.debug(f"Candid: {score:6.3f} {describe(elem)} link density {ld:.3f} -> {score*(1-ld):6.3f}")
             candidate['content_score'] *= (1 - ld)
 
         return candidates
@@ -297,7 +303,7 @@ class Document:
         for elem in self.html.iter():
             if elem in self.keep_elements:
                 continue
-            s = "%s %s" % (elem.get('class', ''), elem.get('id', ''))
+            s = "{} {}".format(elem.get('class', ''), elem.get('id', ''))
             # self.debug(s)
             if REGEXES['unlikelyCandidatesRe'].search(s) and (not REGEXES['okMaybeItsACandidateRe'].search(s)) and elem.tag != 'body':
                 self.debug("Removing unlikely candidate - %s" % describe(elem))
@@ -306,10 +312,10 @@ class Document:
     def transform_misused_divs_into_paragraphs(self):
         for elem in self.tags(self.html, 'div'):
             # transform <div>s that do not contain other block elements into <p>s
-            if not REGEXES['divToPElementsRe'].search(unicode(''.join(map(tostring, list(elem))))):
+            if not REGEXES['divToPElementsRe'].search(str(''.join(map(tounicode, list(elem))))):
                 # self.debug("Altering %s to p" % (describe(elem)))
                 elem.tag = "p"
-                # print "Fixed element "+describe(elem)
+                # print("Fixed element "+describe(elem))
 
         for elem in self.tags(self.html, 'div'):
             if elem.text and elem.text.strip():
@@ -317,7 +323,7 @@ class Document:
                 p.text = elem.text
                 elem.text = None
                 elem.insert(0, p)
-                # print "Appended "+tounicode(p)+" to "+describe(elem)
+                # print("Appended "+tounicode(p)+" to "+describe(elem))
 
             for pos, child in reversed(list(enumerate(elem))):
                 if child.tail and child.tail.strip():
@@ -325,20 +331,18 @@ class Document:
                     p.text = child.tail
                     child.tail = None
                     elem.insert(pos + 1, p)
-                    # print "Inserted "+tounicode(p)+" to "+describe(elem)
+                    # print("Inserted "+tounicode(p)+" to "+describe(elem))
                 if child.tag == 'br':
-                    # print 'Dropped <br> at '+describe(elem)
+                    # print('Dropped <br> at '+describe(elem))
                     child.drop_tree()
 
     def tags(self, node, *tag_names):
         for tag_name in tag_names:
-            for e in node.findall('.//%s' % tag_name):
-                yield e
+            yield from node.findall('.//%s' % tag_name)
 
     def reverse_tags(self, node, *tag_names):
         for tag_name in tag_names:
-            for e in reversed(node.findall('.//%s' % tag_name)):
-                yield e
+            yield from reversed(node.findall('.//%s' % tag_name))
 
     def sanitize(self, node, candidates):
         MIN_LEN = self.options.get('min_text_length', self.TEXT_LENGTH_THRESHOLD)
@@ -356,7 +360,7 @@ class Document:
             weight = self.class_weight(el)
             if el in candidates:
                 content_score = candidates[el]['content_score']
-                # print '!',el, '-> %6.3f' % content_score
+                # print('!',el, '-> %6.3f' % content_score)
             else:
                 content_score = 0
             tag = el.tag
@@ -403,10 +407,10 @@ class Document:
                     reason = "too short content length %s without a single image" % content_length
                     to_remove = True
                 elif weight < 25 and link_density > 0.2:
-                    reason = "too many links %.3f for its weight %s" % (link_density, weight)
+                    reason = f"too many links {link_density:.3f} for its weight {weight}"
                     to_remove = True
                 elif weight >= 25 and link_density > 0.5:
-                    reason = "too many links %.3f for its weight %s" % (link_density, weight)
+                    reason = f"too many links {link_density:.3f} for its weight {weight}"
                     to_remove = True
                 elif (counts["embed"] == 1 and content_length < 75) or counts["embed"] > 1:
                     reason = "<embed>s with too short content length, or too many <embed>s"
@@ -438,7 +442,7 @@ class Document:
                         # self.debug(sib.text_content())
                         sib_content_length = text_length(sib)
                         if sib_content_length:
-                            i =+ 1
+                            i += 1
                             siblings.append(sib_content_length)
                             if i == x:
                                 break
@@ -460,12 +464,13 @@ class Document:
                 if to_remove:
                     self.debug("Cleaned %6.3f %s with weight %s cause it has %s." %
                         (content_score, describe(el), weight, reason))
-                    # print tounicode(el)
+                    # print(tounicode(el))
                     # self.debug("pname %s pweight %.3f" %(pname, pweight))
                     el.drop_tree()
 
         return clean_attributes(tounicode(node))
 
+
 def option_parser():
     from calibre.utils.config import OptionParser
     parser = OptionParser(usage='%prog: [options] file')
@@ -478,6 +483,7 @@ def option_parser():
 
     return parser
 
+
 def main():
     from calibre.utils.logging import default_log
     parser = option_parser()
@@ -493,10 +499,11 @@ def main():
     enc = sys.__stdout__.encoding or 'utf-8'
     if options.verbose:
         default_log.filter_level = default_log.DEBUG
-    print (Document(raw, default_log,
+    print(Document(raw, default_log,
             debug=options.verbose,
             keep_elements=options.keep_elements).summary().encode(enc,
                 'replace'))
 
+
 if __name__ == '__main__':
     main()
