@@ -10,9 +10,11 @@
 #                                                                       #
 #                                                                       #
 #########################################################################
-import sys,os
+import os
+import sys
 
 from calibre.ebooks.rtf2xml import copy
+
 from . import open_for_read, open_for_write
 
 
