@@ -1,4 +1,3 @@
-from __future__ import unicode_literals, absolute_import, print_function, division
 #########################################################################
 #                                                                       #
 #                                                                       #
@@ -11,9 +10,12 @@ from __future__ import unicode_literals, absolute_import, print_function, divisi
 #                                                                       #
 #                                                                       #
 #########################################################################
-import sys, os
-from calibre.ebooks.rtf2xml import copy, border_parse
+import os
+import sys
+
+from calibre.ebooks.rtf2xml import border_parse, copy
 from calibre.ptempfile import better_mktemp
+
 from . import open_for_read, open_for_write
 
 
@@ -89,7 +91,7 @@ class Styles:
         'default-ta'    : 'default-tab',
         'align_____'    : 'align',
         'widow-cntr'    : 'widow-control',
-        # page fomratting mixed in! (Just in older RTF?)
+        # page formatting mixed in! (Just in older RTF?)
         'margin-lef'    :       'left-indent',
         'margin-rig'    :       'right-indent',
         'margin-bot'    :       'space-after',
@@ -122,7 +124,7 @@ class Styles:
         'list-simpi'    : 'list-simple',
         'list-conti'    : 'list-continue',
         'list-hang_'    : 'list-hang',
-        # 'list-tebef'    :	'list-text-before',
+        # 'list-tebef'    : 'list-text-before',
         # 'list-level'    : 'level',
         'list-id___'    : 'list-id',
         'list-start'    : 'list-start',
@@ -527,7 +529,7 @@ class Styles:
             style. I accomplish this by simply looking up the value of 15 in
             the styles table.
             Use two loops. First, check all the paragraph styles. Then check
-            all the characer styles.
+            all the character styles.
             The inner loop: first check 'next-style', then check 'based-on-style'.
             Make sure values exist for the keys to avoid the nasty keyerror message.
         """
@@ -550,7 +552,7 @@ class Styles:
                                 pass
                             else:
                                 if self.__run_level > 4:
-                                    msg = '%s %s is based on %s\n' % (type, key, value)
+                                    msg = f'{type} {key} is based on {value}\n'
                                     msg = 'There is no style with %s\n' % value
                                     raise self.__bug_handler(msg)
                             del self.__styles_dict[type][key][style]
@@ -581,13 +583,13 @@ class Styles:
             style_numbers = self.__styles_dict[type].keys()
             for num in style_numbers:
                 self.__write_obj.write(
-                'mi<tg<empty-att_<%s-style-in-table<num>%s' % (prefix, num)
+                f'mi<tg<empty-att_<{prefix}-style-in-table<num>{num}'
                 )
                 attributes = self.__styles_dict[type][num].keys()
                 for att in attributes:
                     this_value = self.__styles_dict[type][num][att]
                     self.__write_obj.write(
-                        '<%s>%s' % (att, this_value)
+                        f'<{att}>{this_value}'
                         )
                 self.__write_obj.write('\n')
             self.__write_obj.write(
@@ -629,7 +631,7 @@ class Styles:
         Returns:
             nothing
         Logic:
-            Check the line for the beginning of an individaul style. If it is
+            Check the line for the beginning of an individual style. If it is
             not found, simply print out the line.
         """
         action = self.__state_dict.get(self.__token_info)
@@ -662,7 +664,7 @@ class Styles:
             value = None
         if value:
             self.__write_obj.write(
-            'cw<ss<%s-style<nu<%s\n' % (prefix, value)
+            f'cw<ss<{prefix}-style<nu<{value}\n'
             )
         else:
             self.__write_obj.write(
@@ -698,7 +700,7 @@ class Styles:
             beginning of the style table.
             If the state is in the style table, create the style dictionary
             and print out the tags.
-            If the state if afer the style table, look for lines with style
+            If the state if after the style table, look for lines with style
             info, and substitute the number with the name of the style.
         """
         self.__initiate_values()
