@@ -9,25 +9,47 @@ import posixpath
 import sys
 import weakref
 from contextlib import suppress
-from functools import partial, lru_cache
+from functools import lru_cache, partial
+
 from qt.core import (
-    QAction, QCoreApplication, QDialog, QDialogButtonBox, QGridLayout, QIcon,
-    QInputDialog, QLabel, QLineEdit, QMenu, QSize, Qt, QTimer, QToolButton,
-    QVBoxLayout, pyqtSignal
+    QAction,
+    QCoreApplication,
+    QDialog,
+    QDialogButtonBox,
+    QGridLayout,
+    QIcon,
+    QInputDialog,
+    QLabel,
+    QLineEdit,
+    QMenu,
+    QSize,
+    Qt,
+    QTimer,
+    QToolButton,
+    QVBoxLayout,
+    pyqtSignal,
 )
 
 from calibre import isbytestring, sanitize_file_name
-from calibre.constants import (
-    config_dir, filesystem_encoding, get_portable_base, isportable, iswindows
-)
+from calibre.constants import config_dir, filesystem_encoding, get_portable_base, isportable, iswindows
 from calibre.gui2 import (
-    Dispatcher, choose_dir, choose_images, error_dialog, gprefs, info_dialog,
-    open_local_file, pixmap_to_data, question_dialog, warning_dialog
+    Dispatcher,
+    choose_dir,
+    choose_images,
+    error_dialog,
+    gprefs,
+    info_dialog,
+    open_local_file,
+    pixmap_to_data,
+    question_dialog,
+    warning_dialog,
 )
 from calibre.gui2.actions import InterfaceAction
 from calibre.library import current_library_name
+from calibre.startup import connect_lambda
 from calibre.utils.config import prefs, tweaks
 from calibre.utils.icu import sort_key
+from calibre.utils.localization import ngettext
 
 
 def db_class():
@@ -558,7 +580,8 @@ class ChooseLibraryAction(InterfaceAction):
         newname, ok = QInputDialog.getText(self.gui, _('Rename') + ' ' + old_name,
                 '<p>'+_(
                     'Choose a new name for the library <b>%s</b>. ')%name + '<p>'+_(
-                    'Note that the actual library folder will be renamed.'),
+                        'Note that the actual library folder will be renamed.') + '<p>' + _(
+                            'WARNING: This means that any calibre:// URLs that point to things in this library will stop working.'),
                 text=old_name)
         newname = sanitize_file_name(str(newname))
         if not ok or not newname or newname == old_name:
@@ -655,6 +678,7 @@ class ChooseLibraryAction(InterfaceAction):
         db = m.db
         db.prefs.disable_setting = True
         library_path = db.library_path
+        before = db.new_api.size_stats()
 
         d = DBCheck(self.gui, db)
         try:
@@ -669,10 +693,22 @@ class ChooseLibraryAction(InterfaceAction):
         if d.rejected:
             return
         if d.error is None:
+            after = self.gui.current_db.new_api.size_stats()
+            det_msg = ''
+            from calibre import human_readable
+            for which, title in {'main': _('books'), 'fts': _('full text search'), 'notes': _('notes')}.items():
+                if which != 'main' and not getattr(d, which).isChecked():
+                    continue
+                det_msg += '\n'
+                if before[which] == after[which]:
+                    det_msg += _('Size of the {} database was unchanged.').format(title)
+                else:
+                    det_msg += _('Size of the {0} database reduced from {1} to {2}.').format(
+                            title, human_readable(before[which]), human_readable(after[which]))
             if not question_dialog(self.gui, _('Success'),
                     _('Found no errors in your calibre library database.'
                         ' Do you want calibre to check if the files in your'
-                        ' library match the information in the database?')):
+                        ' library match the information in the database?'), det_msg=det_msg.strip()):
                 return
         else:
             return error_dialog(self.gui, _('Failed'),
@@ -686,9 +722,12 @@ class ChooseLibraryAction(InterfaceAction):
             d = CheckLibraryDialog(self.gui, m.db)
 
             if not d.do_exec():
-                info_dialog(self.gui, _('No problems found'),
-                        _('The files in your library match the information '
-                        'in the database.'), show=True)
+                if question_dialog(self.gui, _('No problems found'),
+                        _('The files in your library match the information in the database.\n\n'
+                          "Choose 'Open dialog' to change settings and run the check again."),
+                        yes_text=_('Open dialog'), yes_icon='gear.png', no_icon='ok.png',
+                        no_text=_('Finished')):
+                    d.exec()
         finally:
             self.gui.status_bar.clear_message()
 
