@@ -8,12 +8,35 @@ __docformat__ = 'restructuredtext en'
 import os
 import sys
 from datetime import datetime
+
 from qt.core import (
-    QAbstractTextDocumentLayout, QApplication, QComboBox, QDate, QDateTime,
-    QDateTimeEdit, QDialog, QDoubleSpinBox, QEvent, QFont, QFontInfo, QIcon,
-    QKeySequence, QLineEdit, QLocale, QMenu, QPalette, QSize, QSpinBox, QStyle,
-    QStyledItemDelegate, QStyleOptionComboBox, QStyleOptionSpinBox,
-    QStyleOptionViewItem, Qt, QTextDocument, QUrl,
+    QAbstractTextDocumentLayout,
+    QApplication,
+    QComboBox,
+    QDate,
+    QDateTime,
+    QDateTimeEdit,
+    QDialog,
+    QDoubleSpinBox,
+    QEvent,
+    QFont,
+    QFontInfo,
+    QIcon,
+    QKeySequence,
+    QLineEdit,
+    QLocale,
+    QMenu,
+    QPalette,
+    QSize,
+    QSpinBox,
+    QStyle,
+    QStyledItemDelegate,
+    QStyleOptionComboBox,
+    QStyleOptionSpinBox,
+    QStyleOptionViewItem,
+    Qt,
+    QTextDocument,
+    QUrl,
 )
 
 from calibre.constants import iswindows
@@ -25,16 +48,11 @@ from calibre.gui2.dialogs.tag_editor import TagEditor
 from calibre.gui2.languages import LanguagesEdit
 from calibre.gui2.markdown_editor import MarkdownEditDialog
 from calibre.gui2.widgets import EnLineEdit
-from calibre.gui2.widgets2 import (
-    DateTimeEdit as DateTimeEditBase, RatingEditor,
-    populate_standard_spinbox_context_menu,
-)
+from calibre.gui2.widgets2 import DateTimeEdit as DateTimeEditBase
+from calibre.gui2.widgets2 import RatingEditor, populate_standard_spinbox_context_menu
 from calibre.library.comments import markdown
 from calibre.utils.config import tweaks
-from calibre.utils.date import (
-    format_date, internal_iso_format_string, is_date_undefined, now, qt_from_dt,
-    qt_to_dt,
-)
+from calibre.utils.date import format_date, internal_iso_format_string, is_date_undefined, now, qt_from_dt, qt_to_dt
 from calibre.utils.icu import sort_key
 
 
@@ -82,11 +100,8 @@ class UpdateEditorGeometry:
                 is_pin_view = True
                 break
             p = p.parent()
-        if is_pin_view:
-            max_width = pin_view.horizontalScrollBar().geometry().width()
-        else:
-            view = self.table_widget
-            max_width = view.horizontalScrollBar().geometry().width() - view.verticalHeader().width()
+
+        max_width = (pin_view if is_pin_view else self.table_widget).viewport().rect().width()
         # What we have to display might not fit. If so, adjust down
         new_width = new_width if new_width < max_width else max_width
 
@@ -121,7 +136,7 @@ class UpdateEditorGeometry:
 
 class EditableTextDelegate:
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         n = editor.metaObject().userProperty().name()
         editor.setProperty(n, get_val_for_textlike_columns(index))
 
@@ -187,10 +202,58 @@ def get_val_for_textlike_columns(index_):
 # }}}
 
 
-class RatingDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class StyledItemDelegate(QStyledItemDelegate):
+
+    '''
+    When closing an editor and opening another, Qt sometimes picks what appears
+    to be a random line and column for the second editor. This function checks
+    that the current index for a new editor is the same as the current view. If
+    it isn't then the editor shouldn't be opened.
+
+    Set the flag ignore_kb_mods_on_edit before opening an editor if you don't
+    want keyboard modifiers taken into account, for example when using Shift-Tab
+    as a backtab when editing cells. This prevents opening dialogs by mistake.
+    See giu2.library.views.closeEditor() for an example.
+    '''
 
     def __init__(self, *args, **kwargs):
-        QStyledItemDelegate.__init__(self, *args)
+        super().__init__(*args, **kwargs)
+        self.table_widget = args[0]
+        # Set this to True here. It is up the the subclasses to set it to False if needed.
+        self.is_editable_with_tab = True
+        self.ignore_kb_mods_on_edit = False
+
+    def createEditor(self, parent, option, index):
+        if self.table_widget.currentIndex() != index:
+            idx = self.table_widget.currentIndex()
+            print(f'createEditor idx err: delegate={self.__class__.__name__}. '
+                  f'cur idx=({idx.row()}, {idx.column()}), '
+                  f'given idx=({index.row()}, {index.column()})')
+            return None
+        e = self.create_editor(parent, option, index)
+        return e
+
+    def setEditorData(self, editor, index):
+        # This method exists because of the ignore_kb_mods_on_edit flag. The
+        # flag is cleared after the editor data is set, in set_editor_data. It
+        # is possible that the subclass doesn't implement set_editor_data(). I
+        # can't find a case where this is true, but just in case call the
+        # default.
+        if hasattr(self, 'set_editor_data'):
+            self.set_editor_data(editor, index)
+        else:
+            super().setEditorData(editor, index)
+        self.ignore_kb_mods_on_edit = False
+
+    def create_editor(self, parent, option, index):
+        # Must be overridden by the "real" createEditor
+        raise NotImplementedError
+
+
+class RatingDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
+
+    def __init__(self, *args, **kwargs):
+        StyledItemDelegate.__init__(self, *args)
         self.is_half_star = kwargs.get('is_half_star', False)
         self.table_widget = args[0]
         self.rf = QFont(rating_font())
@@ -206,10 +269,10 @@ class RatingDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
     def displayText(self, value, locale):
         return rating_to_stars(value, self.is_half_star)
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         return RatingEditor(parent, is_half_star=self.is_half_star)
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
             val = 0
         else:
@@ -223,21 +286,21 @@ class RatingDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
     def sizeHint(self, option, index):
         option.font = self.rf
         option.textElideMode = self.em
-        return QStyledItemDelegate.sizeHint(self, option, index)
+        return StyledItemDelegate.sizeHint(self, option, index)
 
     def paint(self, painter, option, index):
         option.font = self.rf
         option.textElideMode = self.em
-        return QStyledItemDelegate.paint(self, painter, option, index)
+        return StyledItemDelegate.paint(self, painter, option, index)
 
 # }}}
 
 
-class DateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class DateDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     def __init__(self, parent, tweak_name='gui_timestamp_display_format',
             default_format='dd MMM yyyy'):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
         self.tweak_name = tweak_name
         self.format = tweaks[self.tweak_name]
@@ -250,13 +313,13 @@ class DateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
             return ''
         return format_date(d, self.format)
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         return DateTimeEdit(parent, self.format)
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
             val = UNDEFINED_QDATETIME
-        elif check_key_modifier(Qt.KeyboardModifier.ShiftModifier | Qt.KeyboardModifier.ControlModifier):
+        elif not self.ignore_kb_mods_on_edit and check_key_modifier(Qt.KeyboardModifier.ShiftModifier):
             val = now()
         else:
             val = index.data(Qt.ItemDataRole.EditRole)
@@ -269,10 +332,10 @@ class DateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class PubDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class PubDateDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     def __init__(self, *args, **kwargs):
-        QStyledItemDelegate.__init__(self, *args, **kwargs)
+        StyledItemDelegate.__init__(self, *args, **kwargs)
         self.format = tweaks['gui_pubdate_display_format']
         self.table_widget = args[0]
         if self.format is None:
@@ -284,14 +347,14 @@ class PubDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
             return ''
         return format_date(d, self.format)
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         return DateTimeEdit(parent, self.format)
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         val = index.data(Qt.ItemDataRole.EditRole)
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
             val = UNDEFINED_QDATETIME
-        elif check_key_modifier(Qt.KeyboardModifier.ShiftModifier | Qt.KeyboardModifier.ControlModifier):
+        elif not self.ignore_kb_mods_on_edit and check_key_modifier(Qt.KeyboardModifier.ShiftModifier):
             val = now()
         elif is_date_undefined(val):
             val = QDate.currentDate()
@@ -302,7 +365,7 @@ class PubDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class TextDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDelegate):  # {{{
+class TextDelegate(StyledItemDelegate, UpdateEditorGeometry, EditableTextDelegate):  # {{{
 
     use_title_sort = False
 
@@ -312,14 +375,14 @@ class TextDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDelega
         of text items to auto-complete with. If the function is None no
         auto-complete will be used.
         '''
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
         self.auto_complete_function = None
 
     def set_auto_complete_function(self, f):
         self.auto_complete_function = f
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         if self.auto_complete_function:
             if self.use_title_sort:
                 editor = EditWithComplete(parent, sort_func=title_sort)
@@ -338,7 +401,7 @@ class TextDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDelega
             val = editor.lineEdit().text()
             model.setData(index, (val), Qt.ItemDataRole.EditRole)
         else:
-            QStyledItemDelegate.setModelData(self, editor, model, index)
+            StyledItemDelegate.setModelData(self, editor, model, index)
 
 # }}}
 
@@ -353,10 +416,10 @@ class SeriesDelegate(TextDelegate):  # {{{
 # }}}
 
 
-class CompleteDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDelegate):  # {{{
+class CompleteDelegate(StyledItemDelegate, UpdateEditorGeometry, EditableTextDelegate):  # {{{
 
     def __init__(self, parent, sep, items_func_name, space_before_sep=False):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.sep = sep
         self.items_func_name = items_func_name
         self.space_before_sep = space_before_sep
@@ -365,12 +428,12 @@ class CompleteDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDe
     def set_database(self, db):
         self.db = db
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         if self.db and hasattr(self.db, self.items_func_name):
             m = index.model()
             col = m.column_map[index.column()]
             # If shifted, bring up the tag editor instead of the line editor.
-            if check_key_modifier(Qt.KeyboardModifier.ShiftModifier) and col != 'authors':
+            if not self.ignore_kb_mods_on_edit and check_key_modifier(Qt.KeyboardModifier.ShiftModifier) and col != 'authors':
                 key = col if m.is_custom_column(col) else None
                 d = TagEditor(parent, self.db, m.id(index.row()), key=key)
                 if d.exec() == QDialog.DialogCode.Accepted:
@@ -399,22 +462,22 @@ class CompleteDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDe
             val = editor.lineEdit().text()
             model.setData(index, (val), Qt.ItemDataRole.EditRole)
         else:
-            QStyledItemDelegate.setModelData(self, editor, model, index)
+            StyledItemDelegate.setModelData(self, editor, model, index)
 # }}}
 
 
-class LanguagesDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class LanguagesDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         editor = LanguagesEdit(parent=parent)
         editor.init_langs(index.model().db)
         return editor
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         editor.show_initial_value(get_val_for_textlike_columns(index))
 
     def setModelData(self, editor, model, index):
@@ -424,7 +487,7 @@ class LanguagesDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class CcDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class CcDateDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     '''
     Delegate for custom columns dates. Because this delegate stores the
@@ -433,7 +496,7 @@ class CcDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
     '''
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
 
     def set_format(self, _format):
@@ -450,13 +513,13 @@ class CcDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
             return ''
         return format_date(d, self.format)
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         return DateTimeEdit(parent, self.format)
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
             val = UNDEFINED_QDATETIME
-        elif check_key_modifier(Qt.KeyboardModifier.ShiftModifier | Qt.KeyboardModifier.ControlModifier):
+        elif not self.ignore_kb_mods_on_edit and check_key_modifier(Qt.KeyboardModifier.ShiftModifier):
             val = now()
         else:
             val = index.data(Qt.ItemDataRole.EditRole)
@@ -475,7 +538,7 @@ class CcDateDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class CcTextDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDelegate):  # {{{
+class CcTextDelegate(StyledItemDelegate, UpdateEditorGeometry, EditableTextDelegate):  # {{{
 
     '''
     Delegate for text data.
@@ -483,10 +546,10 @@ class CcTextDelegate(QStyledItemDelegate, UpdateEditorGeometry, EditableTextDele
     use_title_sort = False
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         m = index.model()
         col = m.column_map[index.column()]
         key = m.db.field_metadata.key_to_label(col)
@@ -524,17 +587,19 @@ class CcSeriesDelegate(CcTextDelegate):  # {{{
 # }}}
 
 
-class CcLongTextDelegate(QStyledItemDelegate):  # {{{
+class CcLongTextDelegate(StyledItemDelegate):  # {{{
 
     '''
     Delegate for comments data.
     '''
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
+        self.table_widget = parent
         self.document = QTextDocument()
+        self.is_editable_with_tab = False
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         m = index.model()
         col = m.column_map[index.column()]
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
@@ -551,7 +616,7 @@ class CcLongTextDelegate(QStyledItemDelegate):  # {{{
 # }}}
 
 
-class CcMarkdownDelegate(QStyledItemDelegate):  # {{{
+class CcMarkdownDelegate(StyledItemDelegate):  # {{{
 
     '''
     Delegate for markdown data.
@@ -559,7 +624,9 @@ class CcMarkdownDelegate(QStyledItemDelegate):  # {{{
 
     def __init__(self, parent):
         super().__init__(parent)
+        self.table_widget = parent
         self.document = QTextDocument()
+        self.is_editable_with_tab = False
 
     def paint(self, painter, option, index):
         self.initStyleOption(option, index)
@@ -583,7 +650,7 @@ class CcMarkdownDelegate(QStyledItemDelegate):  # {{{
         self.document.documentLayout().draw(painter, ctx)
         painter.restore()
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         m = index.model()
         col = m.column_map[index.column()]
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
@@ -604,17 +671,17 @@ class CcMarkdownDelegate(QStyledItemDelegate):  # {{{
 # }}}
 
 
-class CcNumberDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class CcNumberDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     '''
     Delegate for text/int/float data.
     '''
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         m = index.model()
         col = m.column_map[index.column()]
         if m.custom_columns[col]['datatype'] == 'int':
@@ -636,7 +703,7 @@ class CcNumberDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
         model.setData(index, (val), Qt.ItemDataRole.EditRole)
         editor.adjustSize()
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         m = index.model()
         val = m.db.data[index.row()][m.custom_columns[m.column_map[index.column()]]['rec_index']]
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
@@ -655,18 +722,18 @@ class CcNumberDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class CcEnumDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class CcEnumDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     '''
     Delegate for text/int/float data.
     '''
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
         self.longest_text = ''
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         m = index.model()
         col = m.column_map[index.column()]
         editor = DelegateCB(parent)
@@ -690,7 +757,7 @@ class CcEnumDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
                                    self.longest_text + 'M')
         return srect.width()
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         m = index.model()
         val = m.db.data[index.row()][m.custom_columns[m.column_map[index.column()]]['rec_index']]
         if val is None or check_key_modifier(Qt.KeyboardModifier.ControlModifier):
@@ -703,15 +770,17 @@ class CcEnumDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class CcCommentsDelegate(QStyledItemDelegate):  # {{{
+class CcCommentsDelegate(StyledItemDelegate):  # {{{
 
     '''
     Delegate for comments data.
     '''
 
     def __init__(self, parent):
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
+        self.table_widget = parent
         self.document = QTextDocument()
+        self.is_editable_with_tab = False
 
     def paint(self, painter, option, index):
         self.initStyleOption(option, index)
@@ -735,7 +804,7 @@ class CcCommentsDelegate(QStyledItemDelegate):  # {{{
         self.document.documentLayout().draw(painter, ctx)
         painter.restore()
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         m = index.model()
         col = m.column_map[index.column()]
         if check_key_modifier(Qt.KeyboardModifier.ControlModifier):
@@ -765,17 +834,17 @@ class DelegateCB(QComboBox):  # {{{
 # }}}
 
 
-class CcBoolDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
+class CcBoolDelegate(StyledItemDelegate, UpdateEditorGeometry):  # {{{
 
     def __init__(self, parent):
         '''
         Delegate for custom_column bool data.
         '''
         self.nuke_option_data = False
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
         self.table_widget = parent
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         editor = DelegateCB(parent)
         items = [_('Yes'), _('No'), _('Undefined')]
         icons = ['ok.png', 'list_remove.png', 'blank.png']
@@ -798,7 +867,7 @@ class CcBoolDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
         val = {0:True, 1:False, 2:None}[editor.currentIndex()]
         model.setData(index, val, Qt.ItemDataRole.EditRole)
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         m = index.model()
         val = m.db.data[index.row()][m.custom_columns[m.column_map[index.column()]]['rec_index']]
         if not m.db.new_api.pref('bools_are_tristate'):
@@ -829,16 +898,18 @@ class CcBoolDelegate(QStyledItemDelegate, UpdateEditorGeometry):  # {{{
 # }}}
 
 
-class CcTemplateDelegate(QStyledItemDelegate):  # {{{
+class CcTemplateDelegate(StyledItemDelegate):  # {{{
 
     def __init__(self, parent):
         '''
         Delegate for composite custom_columns.
         '''
-        QStyledItemDelegate.__init__(self, parent)
+        StyledItemDelegate.__init__(self, parent)
+        self.table_widget = parent
         self.disallow_edit = gprefs['edit_metadata_templates_only_F2_on_booklist']
+        self.is_editable_with_tab = False
 
-    def createEditor(self, parent, option, index):
+    def create_editor(self, parent, option, index):
         if self.disallow_edit:
             editor = QLineEdit(parent)
             editor.setText(_('Template editing disabled'))
@@ -860,7 +931,7 @@ class CcTemplateDelegate(QStyledItemDelegate):  # {{{
             m.setData(index, (editor.rule[1]), Qt.ItemDataRole.EditRole)
         return None
 
-    def setEditorData(self, editor, index):
+    def set_editor_data(self, editor, index):
         editor.setText('editing templates disabled')
         editor.setReadOnly(True)
 
