@@ -632,10 +632,13 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('dnd_merge', gprefs)
         r('wrap_toolbar_text', gprefs, restart_required=True)
         r('show_layout_buttons', gprefs)
+        r('show_sb_all_actions_button', gprefs)
+        # r('show_sb_preference_button', gprefs)
         r('row_numbers_in_book_list', gprefs)
         r('tag_browser_old_look', gprefs)
         r('tag_browser_hide_empty_categories', gprefs)
         r('tag_browser_always_autocollapse', gprefs)
+        r('tag_browser_restore_tree_expansion', gprefs)
         r('tag_browser_show_tooltips', gprefs)
         r('tag_browser_allow_keyboard_focus', gprefs)
         r('bd_show_cover', gprefs)
@@ -714,7 +717,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('cb_double_click_to_activate', gprefs)
 
         choices = [(_('Off'), 'off'), (_('Small'), 'small'),
-            (_('Medium'), 'medium'), (_('Large'), 'large')]
+            (_('Medium-small'), 'mid-small'), (_('Medium'), 'medium'), (_('Large'), 'large')]
         r('toolbar_icon_size', gprefs, choices=choices)
 
         choices = [(_('If there is enough room'), 'auto'), (_('Always'), 'always'),
@@ -1276,6 +1279,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         qv = get_quickview_action_plugin()
         if qv:
             qv.refill_quickview()
+        gui.sb_all_gui_actions_button.setVisible(gprefs['show_sb_all_actions_button'])
+        # gui.sb_preferences_button.setVisible(gprefs['show_sb_preference_button'])
 
 
 if __name__ == '__main__':
