@@ -8,9 +8,24 @@ __docformat__ = 'restructuredtext en'
 import copy
 import textwrap
 from functools import partial
+
 from qt.core import (
-    QAction, QActionGroup, QComboBox, QDialog, QFrame, QHBoxLayout, QIcon, QLabel,
-    QLineEdit, QMenu, QSizePolicy, Qt, QTimer, QToolButton, QVBoxLayout, QWidget,
+    QAction,
+    QActionGroup,
+    QComboBox,
+    QDialog,
+    QFrame,
+    QHBoxLayout,
+    QIcon,
+    QLabel,
+    QLineEdit,
+    QMenu,
+    QSizePolicy,
+    Qt,
+    QTimer,
+    QToolButton,
+    QVBoxLayout,
+    QWidget,
     pyqtSignal,
 )
 
@@ -86,7 +101,7 @@ class TagBrowserMixin:  # {{{
                 current_cat = model.column_map[col]
                 if current_cat in ('authors', 'series', 'publisher', 'tags') or current_cat in cust_cats:
                     cdn = cat_display_name(current_cat) or current_cat
-                    m.addAction(get_icon(current_cat), cdn, menu_func(current_cat, None))
+                    m.addAction(get_icon(current_cat), cdn.replace('&', '&&'), menu_func(current_cat, None))
                     proxy_md = db.new_api.get_proxy_metadata(db.id(idx.row()))
                     items = proxy_md.get(current_cat)
                     if isinstance(items, str):
@@ -115,7 +130,7 @@ class TagBrowserMixin:  # {{{
             for cat in sorted(cust_cats, key=lambda v: sort_key(cat_display_name(v))):
                 if cat == current_cat:
                     continue
-                m.addAction(get_icon(cat), cat_display_name(cat), menu_func(cat, None))
+                m.addAction(get_icon(cat), cat_display_name(cat).replace('&', '&&'), menu_func(cat, None))
 
     def init_tag_browser_mixin(self, db):
         self.library_view.model().count_changed_signal.connect(self.tags_view.recount_with_position_based_index)
@@ -794,17 +809,24 @@ class TagBrowserWidget(QFrame):  # {{{
                 action=ac, group=_('Tag browser'))
         # Show/hide notes icon
         l.m.show_notes_icon_action = ac = l.m.addAction(QIcon.ic('notes.png'), 'notes icon')
-        ac.triggered.connect(self.toggle_notes_icon)
+        ac.triggered.connect(self.toggle_notes)
         parent.keyboard.register_shortcut('tag browser toggle notes',
                 _('Toggle notes icons'), default_keys=(),
                 action=ac, group=_('Tag browser'))
         # Show/hide links icon
         l.m.show_links_icon_action = ac = l.m.addAction(QIcon.ic('external-link.png'), 'links icon')
-        ac.triggered.connect(self.toggle_links_icon)
+        ac.triggered.connect(self.toggle_links)
         parent.keyboard.register_shortcut('tag browser toggle links',
                 _('Toggle links icons'), default_keys=(),
                 action=ac, group=_('Tag browser'))
 
+        # Show/hide empty categories
+        l.m.show_empty_categories_action = ac = l.m.addAction(QIcon.ic('external-link.png'), 'links icon')
+        ac.triggered.connect(self.toggle_show_empty_categories)
+        parent.keyboard.register_shortcut('tag browser show empty categories',
+                _('Show empty categories (columns)'), default_keys=(),
+                action=ac, group=_('Tag browser'))
+
         sb = l.m.addAction(QIcon.ic('sort.png'), _('Sort by'))
         sb.m = l.sort_menu = QMenu(l.m)
         sb.setMenu(sb.m)
@@ -814,6 +836,10 @@ class TagBrowserWidget(QFrame):  # {{{
         for i, x in enumerate((_('Name'), _('Number of books'),
                   _('Average rating'))):
             a = sb.m.addAction(x)
+            parent.keyboard.register_shortcut(
+                    f"tag browser sort by {('notes', 'number of books', 'average rating')[i]}",
+                    (_('Sort by name'), _('Sort by number of books'), _('Sort by average rating'))[i],
+                    default_keys=(), action=a, group=_('Tag browser'))
             sb.bg.addAction(a)
             a.setCheckable(True)
             if i == 0:
@@ -873,17 +899,29 @@ class TagBrowserWidget(QFrame):  # {{{
 
     def about_to_show_configure_menu(self):
         ac = self.alter_tb.m.show_counts_action
-        ac.setText(_('Hide counts') if gprefs['tag_browser_show_counts'] else _('Show counts'))
-        ac.setIcon(QIcon.ic('minus.png') if gprefs['tag_browser_show_counts'] else QIcon.ic('plus.png'))
+        p = gprefs['tag_browser_show_counts']
+        ac.setText(_('Hide counts') if p else _('Show counts'))
+        ac.setIcon(QIcon.ic('minus.png') if p else QIcon.ic('plus.png'))
+
         ac = self.alter_tb.m.show_avg_rating_action
-        ac.setText(_('Hide average rating') if config['show_avg_rating'] else _('Show average rating'))
-        ac.setIcon(QIcon.ic('minus.png' if config['show_avg_rating'] else 'plus.png'))
+        p = config['show_avg_rating']
+        ac.setText(_('Hide average rating') if p else _('Show average rating'))
+        ac.setIcon(QIcon.ic('minus.png' if p else 'plus.png'))
+
         ac = self.alter_tb.m.show_notes_icon_action
-        ac.setText(_('Hide notes icon') if gprefs['show_notes_in_tag_browser'] else _('Show notes icon'))
-        ac.setIcon(QIcon.ic('minus.png' if gprefs['show_notes_in_tag_browser'] else 'plus.png'))
+        p = gprefs['show_notes_in_tag_browser']
+        ac.setText(_('Hide notes icon') if p else _('Show notes icon'))
+        ac.setIcon(QIcon.ic('minus.png' if p else 'plus.png'))
+
         ac = self.alter_tb.m.show_links_icon_action
-        ac.setText(_('Hide links icon') if gprefs['show_links_in_tag_browser'] else _('Show links icon'))
-        ac.setIcon(QIcon.ic('minus.png' if gprefs['show_links_in_tag_browser'] else 'plus.png'))
+        p = gprefs['show_links_in_tag_browser']
+        ac.setText(_('Hide links icon') if p else _('Show links icon'))
+        ac.setIcon(QIcon.ic('minus.png' if p else 'plus.png'))
+
+        ac = self.alter_tb.m.show_empty_categories_action
+        p = self.tags_view.model().prefs['tag_browser_hide_empty_categories']
+        ac.setText(_('Show empty categories (columns)') if p else _('Hide empty categories (columns)'))
+        ac.setIcon(QIcon.ic('plus.png' if p else 'minus.png'))
 
     def filter_book_list(self):
         self.tags_view.model().set_in_tag_browser()
@@ -897,16 +935,38 @@ class TagBrowserWidget(QFrame):  # {{{
         config['show_avg_rating'] ^= True
         self.tags_view.recount_with_position_based_index()
 
-    def toggle_notes_icon(self):
+    def toggle_notes(self):
         gprefs['show_notes_in_tag_browser'] ^= True
         self.tags_view.recount_with_position_based_index()
 
-    def toggle_links_icon(self):
+    def toggle_links(self):
         gprefs['show_links_in_tag_browser'] ^= True
         self.tags_view.recount_with_position_based_index()
 
-    def save_state(self):
-        gprefs.set('tag browser search box visible', self.toggle_search_button.isChecked())
+    def toggle_show_empty_categories(self):
+        self.tags_view.model().prefs['tag_browser_hide_empty_categories'] ^= True
+        self.tags_view.recount_with_position_based_index()
+
+    def save_state(self, gprefs_local=None):
+        if gprefs_local is None:
+            gprefs_local = gprefs
+        gprefs_local.set('tag browser search box visible', self.toggle_search_button.isChecked())
+
+    def restore_expansion_state(self, state):
+        '''
+        Expands the tag browser tree so that the node specified in state is
+        visible. Use get_expansion_state() to get the state. The intent is that
+        a plugin could restore the state in the library_changed() method.
+        '''
+        if state is not None:
+            self.tags_view.restore_expansion(state)
+
+    def get_expansion_state(self):
+        '''
+        Returns the currently expanded node in the tag browser as a string
+        suitable for restoring using restore_expansion_state.
+        '''
+        return self.tags_view.current_expansion
 
     def toggle_item(self):
         self.tags_view.toggle_current_index()
