@@ -1,26 +1,20 @@
-#!/usr/bin/env python2
-# vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+#!/usr/bin/env python
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import cPickle
 from collections import namedtuple
 
-from calibre.utils.localization import canonicalize_lang
+from calibre.utils.localization import canonicalize_lang, load_iso3166
 
 DictionaryLocale = namedtuple('DictionaryLocale', 'langcode countrycode')
 
-ccodes, ccodemap, country_names = None, None, None
 
 def get_codes():
-    global ccodes, ccodemap, country_names
-    if ccodes is None:
-        data = cPickle.loads(P('localization/iso3166.pickle', allow_user_override=False, data=True))
-        ccodes, ccodemap, country_names = data['codes'], data['three_map'], data['names']
-    return ccodes, ccodemap
+    data = load_iso3166()
+    return data['codes'], data['three_map']
+
 
 def parse_lang_code(raw):
     raw = raw or ''
@@ -29,6 +23,9 @@ def parse_lang_code(raw):
     if lc is None:
         raise ValueError('Invalid language code: %r' % raw)
     cc = None
+    for sc in ['Cyrl', 'Latn']:
+        if sc in parts:
+            parts.remove(sc)
     if len(parts) > 1:
         ccodes, ccodemap = get_codes()[:2]
         q = parts[1].upper()
@@ -37,5 +34,3 @@ def parse_lang_code(raw):
         else:
             cc = ccodemap.get(q, None)
     return DictionaryLocale(lc, cc)
-
-
