@@ -1,62 +1,59 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
 
 
-import os, json, struct, hashlib, sys, errno, tempfile, time, shutil, uuid
+import errno
+import hashlib
+import io
+import json
+import os
+import shutil
+import struct
+import sys
+import tempfile
+import time
+import uuid
 from collections import Counter
+from typing import NamedTuple
 
 from calibre import prints
-from calibre.constants import config_dir, iswindows, filesystem_encoding
-from calibre.utils.config_base import prefs, StringConfig, create_global_prefs
+from calibre.constants import config_dir, filesystem_encoding, iswindows
 from calibre.utils.config import JSONConfig
+from calibre.utils.config_base import StringConfig, create_global_prefs, prefs
 from calibre.utils.filenames import samefile
-from polyglot.builtins import iteritems, error_message
+from calibre.utils.localization import _
 from polyglot.binary import as_hex_unicode
-
+from polyglot.builtins import error_message, iteritems
 
 # Export {{{
 
-def send_file(from_obj, to_obj, chunksize=1<<20):
-    m = hashlib.sha1()
-    while True:
-        raw = from_obj.read(chunksize)
-        if not raw:
-            break
-        m.update(raw)
-        to_obj.write(raw)
-    return str(m.hexdigest())
-
-
 class FileDest:
 
     def __init__(self, key, exporter, mtime=None):
         self.exporter, self.key = exporter, key
         self.hasher = hashlib.sha1()
-        self.start_pos = exporter.f.tell()
+        self.start_part_number, self.start_pos = exporter.current_pos()
         self._discard = False
-        self.mtime = None
+        self.mtime = mtime
+        self.size = 0
 
     def discard(self):
         self._discard = True
 
-    def ensure_space(self, size):
-        if size > 0:
-            self.exporter.ensure_space(size)
-            self.start_pos = self.exporter.f.tell()
-
     def write(self, data):
+        self.size += len(data)
+        written = self.exporter.write(data)
+        if len(data) != written:
+            raise RuntimeError(f'Exporter failed to write all data: {len(data)} != {written}')
         self.hasher.update(data)
-        self.exporter.f.write(data)
 
     def flush(self):
         pass
 
     def close(self):
         if not self._discard:
-            size = self.exporter.f.tell() - self.start_pos
             digest = str(self.hasher.hexdigest())
-            self.exporter.file_metadata[self.key] = (len(self.exporter.parts), self.start_pos, size, digest, self.mtime)
+            self.exporter.file_metadata[self.key] = (self.start_part_number, self.start_pos, self.size, digest, self.mtime)
         del self.exporter, self.hasher
 
     def __enter__(self):
@@ -68,17 +65,23 @@ class FileDest:
 
 class Exporter:
 
-    VERSION = 0
+    VERSION = 1
     TAIL_FMT = b'!II?'  # part_num, version, is_last
     MDATA_SZ_FMT = b'!Q'
     EXT = '.calibre-data'
 
-    def __init__(self, path_to_export_dir, part_size=(1 << 30)):
-        self.part_size = part_size
+    @classmethod
+    def tail_size(cls):
+        return struct.calcsize(cls.TAIL_FMT)
+
+    def __init__(self, path_to_export_dir, part_size=None):
+        # default part_size is 1 GB
+        self.part_size = (1 << 30) if part_size is None else part_size
         self.base = os.path.abspath(path_to_export_dir)
-        self.parts = []
-        self.new_part()
+        self.commited_parts = []
+        self.current_part = None
         self.file_metadata = {}
+        self.tail_sz = self.tail_size()
         self.metadata = {'file_metadata': self.file_metadata}
 
     def set_metadata(self, key, val):
@@ -86,47 +89,61 @@ class Exporter:
             raise KeyError('The metadata already contains the key: %s' % key)
         self.metadata[key] = val
 
-    @property
-    def f(self):
-        return self.parts[-1]
+    def current_pos(self):
+        pos = 0
+        if self.current_part is not None:
+            pos = self.current_part.tell()
+            if pos >= self.part_size - self.tail_sz:
+                self.new_part()
+                pos = 0
+        return len(self.commited_parts) + 1, pos
+
+    def write(self, data: bytes) -> int:
+        written = 0
+        data = memoryview(data)
+        while len(data) > 0:
+            if self.current_part is None:
+                self.new_part()
+            max_size = self.part_size - self.tail_sz - self.current_part.tell()
+            if max_size <= 0:
+                self.new_part()
+                max_size = self.part_size - self.tail_sz
+            chunk = data[:max_size]
+            w = self.current_part.write(chunk)
+            data = data[w:]
+            written += w
+        return written
 
     def new_part(self):
-        self.parts.append(open(os.path.join(
-            self.base, 'part-{:04d}{}'.format(len(self.parts) + 1, self.EXT)), 'wb'))
+        self.commit_part()
+        self.current_part = open(os.path.join(
+            self.base, f'part-{len(self.commited_parts) + 1:04d}{self.EXT}'), 'wb')
 
     def commit_part(self, is_last=False):
-        self.f.write(struct.pack(self.TAIL_FMT, len(self.parts), self.VERSION, is_last))
-        self.f.close()
-        self.parts[-1] = self.f.name
-
-    def ensure_space(self, size):
-        try:
-            if size + self.f.tell() < self.part_size:
-                return
-        except AttributeError:
-            raise RuntimeError('This exporter has already been committed, cannot add to it')
-        self.commit_part()
-        self.new_part()
+        if self.current_part is not None:
+            self.current_part.write(struct.pack(self.TAIL_FMT, len(self.commited_parts) + 1, self.VERSION, is_last))
+            self.current_part.close()
+            self.commited_parts.append(self.current_part.name)
+            self.current_part = None
 
     def commit(self):
         raw = json.dumps(self.metadata, ensure_ascii=False)
         if not isinstance(raw, bytes):
             raw = raw.encode('utf-8')
-        self.ensure_space(len(raw))
-        self.f.write(raw)
-        self.f.write(struct.pack(self.MDATA_SZ_FMT, len(raw)))
+        self.new_part()
+        orig, self.part_size = self.part_size, sys.maxsize
+        self.write(raw)
+        self.write(struct.pack(self.MDATA_SZ_FMT, len(raw)))
+        self.part_size = orig
         self.commit_part(is_last=True)
 
     def add_file(self, fileobj, key):
-        fileobj.seek(0, os.SEEK_END)
-        size = fileobj.tell()
-        fileobj.seek(0)
-        self.ensure_space(size)
-        pos = self.f.tell()
-        digest = send_file(fileobj, self.f)
-        size = self.f.tell() - pos
-        mtime = os.fstat(fileobj.fileno()).st_mtime
-        self.file_metadata[key] = (len(self.parts), pos, size, digest, mtime)
+        try:
+            mtime = os.fstat(fileobj.fileno()).st_mtime
+        except (io.UnsupportedOperation, OSError):
+            mtime = None
+        with self.start_file(key, mtime=mtime) as dest:
+            shutil.copyfileobj(fileobj, dest)
 
     def start_file(self, key, mtime=None):
         return FileDest(key, self, mtime=mtime)
@@ -138,15 +155,15 @@ class Exporter:
             for fname in filenames:
                 fpath = os.path.join(dirpath, fname)
                 rpath = os.path.relpath(fpath, path).replace(os.sep, '/')
-                key = '%s:%s' % (pkey, rpath)
+                key = f'{pkey}:{rpath}'
                 try:
-                    with lopen(fpath, 'rb') as f:
+                    with open(fpath, 'rb') as f:
                         self.add_file(f, key)
                 except OSError:
                     if not iswindows:
                         raise
                     time.sleep(1)
-                    with lopen(fpath, 'rb') as f:
+                    with open(fpath, 'rb') as f:
                         self.add_file(f, key)
                 files.append((key, rpath))
 
@@ -170,8 +187,8 @@ def all_known_libraries():
 
 
 def export(destdir, library_paths=None, dbmap=None, progress1=None, progress2=None, abort=None):
-    from calibre.db.cache import Cache
     from calibre.db.backend import DB
+    from calibre.db.cache import Cache
     if library_paths is None:
         library_paths = all_known_libraries()
     dbmap = dbmap or {}
@@ -208,31 +225,135 @@ def export(destdir, library_paths=None, dbmap=None, progress1=None, progress2=No
 
 # Import {{{
 
+class Chunk(NamedTuple):
+    part_num: int
+    pos_in_part: int
+    size: int
+    pos_in_file: int
+
+
+class Pos:
+
+    def __init__(self, part, pos_in_part, size, importer):
+        self.size = size
+        self.pos_in_file = 0
+        self.chunks = chunks = []
+        self.open_part = importer.open_part
+        self.currently_open_part = None
+        self.currently_open_chunk_index = -1
+
+        pos = 0
+        while size > 0:
+            part_size = importer.size_of_part(part)
+            chunk_size = min(size, part_size - pos_in_part)
+            if chunk_size > 0:
+                chunks.append(Chunk(part, pos_in_part, chunk_size, pos))
+                size -= chunk_size
+                pos += chunk_size
+            part += 1
+            pos_in_part = 0
+
+    def close(self):
+        if self.currently_open_part is not None:
+            self.currently_open_part.close()
+            self.currently_open_part = None
+        self.currently_open_chunk_index = -1
+
+    def tell(self) -> int:
+        return self.pos_in_file
+
+    def seek(self, amt, whence=os.SEEK_SET) -> int:
+        if whence == os.SEEK_SET:
+            new_pos_in_file = amt
+        if whence == os.SEEK_END:
+            new_pos_in_file = self.size + amt
+        if whence == os.SEEK_CUR:
+            new_pos_in_file = self.pos_in_file + amt
+        self.pos_in_file = max(0, min(new_pos_in_file, self.size))
+        return self.pos_in_file
+
+    def read(self, size=None):
+        if size is None or size < 0:
+            size = self.size
+        size = min(size, self.size)
+        amt_left = max(0, self.size - self.pos_in_file)
+        amt_to_read = min(amt_left, size)
+        if amt_to_read <= 0:
+            return b''
+        start_chunk = max(0, self.currently_open_chunk_index)
+        num = len(self.chunks)
+        ans = []
+        chunk_idx = -1
+        for i in range(num):
+            chunk_idx = (start_chunk + i) % num
+            chunk = self.chunks[chunk_idx]
+            if chunk.pos_in_file <= self.pos_in_file < chunk.pos_in_file + chunk.size:
+                break
+        else:
+            raise ValueError(f'No chunk found containing {self.pos_in_file=}')
+
+        while amt_to_read > 0:
+            try:
+                chunk = self.chunks[chunk_idx]
+            except IndexError:
+                break
+            ans.append(self._read_chunk(chunk, amt_to_read, chunk_idx))
+            amt_to_read -= len(ans[-1])
+            chunk_idx += 1
+        return b''.join(ans)
+
+    def _read_chunk(self, chunk, size, chunk_idx):
+        if self.currently_open_chunk_index != chunk_idx or self.currently_open_part is None:
+            self.close()
+            self.currently_open_part = self.open_part(chunk.part_num)
+            self.currently_open_chunk_index = chunk_idx
+        offset_from_start_of_chunk = self.pos_in_file - chunk.pos_in_file
+        self.currently_open_part.seek(chunk.pos_in_part + offset_from_start_of_chunk, os.SEEK_SET)
+        size = min(size, chunk.size - offset_from_start_of_chunk)
+        ans = self.currently_open_part.read(size)
+        self.pos_in_file += len(ans)
+        return ans
+
 
 class FileSource:
 
-    def __init__(self, f, size, digest, description, mtime, importer):
-        self.f, self.size, self.digest, self.description = f, size, digest, description
+    def __init__(self, start_partnum, start_pos, size, digest, description, mtime, importer):
+        self.size, self.digest, self.description = size, digest, description
         self.mtime = mtime
-        self.end = f.tell() + size
+        self.start = start_pos
+        self.start_partnum = start_partnum
+        self.pos = Pos(start_partnum, start_pos, size, importer)
         self.hasher = hashlib.sha1()
         self.importer = importer
+        self.check_hash = True
+
+    def seekable(self):
+        return False
+
+    def seek(self, amt, whence=os.SEEK_SET):
+        return self.pos.seek(amt, whence)
+
+    def tell(self):
+        return self.pos.tell()
 
     def read(self, size=None):
-        if size is not None and size < 1:
-            return b''
-        left = self.end - self.f.tell()
-        amt = min(left, size or left)
-        if amt < 1:
-            return b''
-        ans = self.f.read(amt)
-        self.hasher.update(ans)
+        ans = self.pos.read(size)
+        if self.check_hash and ans:
+            self.hasher.update(ans)
         return ans
 
     def close(self):
-        if self.hasher.hexdigest() != self.digest:
+        if self.check_hash and self.hasher.hexdigest() != self.digest:
             self.importer.corrupted_files.append(self.description)
-        self.hasher = self.f = None
+        self.hasher = None
+        self.pos.close()
+        self.pos = None
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *a):
+        self.close()
 
 
 class Importer:
@@ -240,11 +361,14 @@ class Importer:
     def __init__(self, path_to_export_dir):
         self.corrupted_files = []
         part_map = {}
-        tail_size = struct.calcsize(Exporter.TAIL_FMT)
+        self.tail_size = tail_size = struct.calcsize(Exporter.TAIL_FMT)
+        self.version = -1
         for name in os.listdir(path_to_export_dir):
             if name.lower().endswith(Exporter.EXT):
                 path = os.path.join(path_to_export_dir, name)
                 with open(path, 'rb') as f:
+                    f.seek(0, os.SEEK_END)
+                    size_of_part = f.tell()
                     f.seek(-tail_size, os.SEEK_END)
                     raw = f.read()
                 if len(raw) != tail_size:
@@ -254,7 +378,11 @@ class Importer:
                     raise ValueError('The exported data in %s is not valid,'
                                      ' version (%d) is higher than maximum supported version.'
                                      ' You might need to upgrade calibre first.' % (name, version))
-                part_map[part_num] =  path, is_last
+                part_map[part_num] = path, is_last, size_of_part
+                if self.version == -1:
+                    self.version = version
+                if version != self.version:
+                    raise ValueError(f'The exported data in {name} is not valid as it contains a mix of parts with versions: {self.version} and {version}')
         nums = sorted(part_map)
         if not nums:
             raise ValueError('No exported data found in: %s' % path_to_export_dir)
@@ -264,40 +392,47 @@ class Importer:
             raise ValueError('The last part of this exported data set is missing')
         if len(nums) != nums[-1]:
             raise ValueError('There are some parts of the exported data set missing')
-        self.part_map = {num:path for num, (path, is_last) in iteritems(part_map)}
+        self.part_map, self.part_size_map = {}, {}
+        for part_num, (path, is_last, size_of_part) in part_map.items():
+            self.part_map[part_num] = path
+            self.part_size_map[part_num] = size_of_part
         msf = struct.calcsize(Exporter.MDATA_SZ_FMT)
         offset = tail_size + msf
-        with self.part(nums[-1]) as f:
+        with self.open_part(nums[-1]) as f:
             f.seek(-offset, os.SEEK_END)
             sz, = struct.unpack(Exporter.MDATA_SZ_FMT, f.read(msf))
             f.seek(- sz - offset, os.SEEK_END)
             self.metadata = json.loads(f.read(sz))
             self.file_metadata = self.metadata['file_metadata']
 
-    def part(self, num):
-        return lopen(self.part_map[num], 'rb')
+    def size_of_part(self, num):
+        return self.part_size_map[num] - self.tail_size
+
+    def open_part(self, num):
+        return open(self.part_map[num], 'rb')
 
     def start_file(self, key, description):
         partnum, pos, size, digest, mtime = self.file_metadata[key]
-        f = self.part(partnum)
-        f.seek(pos)
-        return FileSource(f, size, digest, description, mtime, self)
+        return FileSource(partnum, pos, size, digest, description, mtime, self)
+
+    def save_file(self, key, description, output_path):
+        with open(output_path, 'wb') as dest, self.start_file(key, description) as src:
+            shutil.copyfileobj(src, dest)
 
     def export_config(self, base_dir, library_usage_stats):
         for key, relpath in self.metadata['config_dir']:
-            f = self.start_file(key, relpath)
-            path = os.path.join(base_dir, relpath.replace('/', os.sep))
-            try:
-                with lopen(path, 'wb') as dest:
-                    shutil.copyfileobj(f, dest)
-            except OSError:
-                os.makedirs(os.path.dirname(path))
-                with lopen(path, 'wb') as dest:
-                    shutil.copyfileobj(f, dest)
-            f.close()
+            with self.start_file(key, relpath) as f:
+                path = os.path.join(base_dir, relpath.replace('/', os.sep))
+                try:
+                    with open(path, 'wb') as dest:
+                        shutil.copyfileobj(f, dest)
+                except OSError:
+                    os.makedirs(os.path.dirname(path))
+                    with open(path, 'wb') as dest:
+                        shutil.copyfileobj(f, dest)
         gpath = os.path.join(base_dir, 'global.py')
         try:
-            with lopen(gpath, 'rb') as f:
+            with open(gpath, 'rb') as f:
                 raw = f.read()
         except OSError:
             raw = b''
@@ -311,7 +446,7 @@ class Importer:
         raw = c.src
         if not isinstance(raw, bytes):
             raw = raw.encode('utf-8')
-        with lopen(gpath, 'wb') as f:
+        with open(gpath, 'wb') as f:
             f.write(raw)
         gprefs = JSONConfig('gui', base_path=base_dir)
         gprefs['library_usage_stats'] = dict(library_usage_stats)
@@ -395,7 +530,7 @@ def input_unicode(prompt):
     return ans
 
 
-def run_exporter(export_dir=None, args=None):
+def run_exporter(export_dir=None, args=None, check_known_libraries=True):
     if args:
         if len(args) < 2:
             raise SystemExit('You must specify the export folder and libraries to export')
@@ -409,8 +544,8 @@ def run_exporter(export_dir=None, args=None):
             libraries = set(all_libraries)
         else:
             libraries = {os.path.normcase(os.path.abspath(os.path.expanduser(path))) for path in args[1:]}
-        if libraries - set(all_libraries):
-            raise SystemExit('Unknown library: ' + tuple(libraries - all_libraries)[0])
+        if check_known_libraries and libraries - set(all_libraries):
+            raise SystemExit('Unknown library: ' + tuple(libraries - set(all_libraries))[0])
         libraries = {p: all_libraries[p] for p in libraries}
         print('Exporting libraries:', ', '.join(sorted(libraries)), 'to:', export_dir)
         export(export_dir, progress1=cli_report, progress2=cli_report, library_paths=libraries)
