@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
 
 
 __license__   = 'GPL v3'
@@ -10,12 +9,13 @@ import os
 from collections import defaultdict
 from threading import Thread
 
-from calibre import walk, prints, as_unicode
-from calibre.constants import (config_dir, iswindows, ismacos, plugins, DEBUG,
-        isworker, filesystem_encoding)
+from calibre import as_unicode, prints, walk
+from calibre.constants import DEBUG, config_dir, filesystem_encoding, ismacos, iswindows, isworker
 from calibre.utils.fonts.metadata import FontMetadata, UnsupportedFont
+from calibre.utils.icu import lower as icu_lower
 from calibre.utils.icu import sort_key
-from polyglot.builtins import itervalues, unicode_type, filter
+from calibre.utils.resources import get_path as P
+from polyglot.builtins import itervalues
 
 
 class NoFonts(ValueError):
@@ -100,9 +100,7 @@ def fc_list():
 
 def font_dirs():
     if iswindows:
-        winutil, err = plugins['winutil']
-        if err:
-            raise RuntimeError('Failed to load winutil: %s'%err)
+        from calibre_extensions import winutil
         paths = {os.path.normcase(r'C:\Windows\Fonts')}
         for which in (winutil.CSIDL_FONTS, winutil.CSIDL_LOCAL_APPDATA, winutil.CSIDL_APPDATA):
             try:
@@ -201,7 +199,7 @@ class FontScanner(Thread):
     CACHE_VERSION = 2
 
     def __init__(self, folders=[], allowed_extensions={'ttf', 'otf'}):
-        Thread.__init__(self)
+        super().__init__(daemon=True)
         self.folders = folders + font_dirs() + [os.path.join(config_dir, 'fonts'),
                 P('fonts/liberation')]
         self.folders = [os.path.normcase(os.path.abspath(f)) for f in
@@ -256,7 +254,7 @@ class FontScanner(Thread):
         path = font_or_path
         if isinstance(font_or_path, dict):
             path = font_or_path['path']
-        with lopen(path, 'rb') as f:
+        with open(path, 'rb') as f:
             return f.read()
 
     def find_font_for_text(self, text, allowed_families={'serif', 'sans-serif'},
@@ -270,10 +268,9 @@ class FontScanner(Thread):
 
         :return: (family name, faces) or None, None
         '''
-        from calibre.utils.fonts.utils import (supports_text,
-                panose_to_css_generic_family, get_printable_characters)
-        if not isinstance(text, unicode_type):
-            raise TypeError(u'%r is not unicode'%text)
+        from calibre.utils.fonts.utils import get_printable_characters, panose_to_css_generic_family, supports_text
+        if not isinstance(text, str):
+            raise TypeError('%r is not unicode'%text)
         text = get_printable_characters(text)
         found = {}
 
@@ -331,7 +328,7 @@ class FontScanner(Thread):
                 continue
             try:
                 files = tuple(walk(folder))
-            except EnvironmentError as e:
+            except OSError as e:
                 if DEBUG:
                     prints('Failed to walk font folder:', folder,
                             as_unicode(e))
@@ -342,9 +339,9 @@ class FontScanner(Thread):
                 candidate = os.path.normcase(os.path.abspath(candidate))
                 try:
                     s = os.stat(candidate)
-                except EnvironmentError:
+                except OSError:
                     continue
-                fileid = '{0}||{1}:{2}'.format(candidate, s.st_size, s.st_mtime)
+                fileid = f'{candidate}||{s.st_size}:{s.st_mtime}'
                 if fileid in cached_fonts:
                     # Use previously cached metadata, since the file size and
                     # last modified timestamp have not changed.
@@ -368,6 +365,7 @@ class FontScanner(Thread):
         self.font_family_map, self.font_families = build_families(self.cached_fonts, self.folders)
 
     def write_cache(self):
+        # writing to the cache is atomic thanks to JSONConfig
         with self.cache:
             self.cache['version'] = self.CACHE_VERSION
             self.cache['fonts'] = self.cached_fonts
@@ -377,7 +375,7 @@ class FontScanner(Thread):
         self.write_cache()
 
     def read_font_metadata(self, path, fileid):
-        with lopen(path, 'rb') as f:
+        with open(path, 'rb') as f:
             try:
                 fm = FontMetadata(f)
             except UnsupportedFont:
